<?php

declare(strict_types=1);

namespace Drupal\graphql_compose_webform\Plugin\GraphQL\DataProducer;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Render\RenderContext;
use Drupal\Core\Render\RendererInterface;
use Drupal\graphql\GraphQL\Execution\FieldContext;
use Drupal\graphql\Plugin\GraphQL\DataProducer\DataProducerPluginBase;
use Drupal\webform\Entity\WebformSubmission as WebformSubmissionEntity;
use Drupal\webform\WebformEntityStorageInterface;
use Drupal\webform\WebformInterface;
use Drupal\webform\WebformSubmissionInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Produces a Webform submission result containing the input data.
 *
 * @DataProducer(
 *   id = "webform_submit",
 *   name = @Translation("Webform submit"),
 *   description = @Translation("Create a webform submission."),
 *   produces = @ContextDefinition("any",
 *     label = @Translation("Webform submission result"),
 *     required = FALSE
 *   ),
 *   consumes = {
 *     "id" = @ContextDefinition("string",
 *       label = @Translation("Webform ID")
 *     ),
 *     "data" = @ContextDefinition("any",
 *       label = @Translation("Array of WebformSubmissionElement objects.")
 *     ),
 *   }
 * )
 */
class WebformSubmission extends DataProducerPluginBase implements ContainerFactoryPluginInterface {

  public function __construct(
    array $configuration,
    $pluginId,
    $pluginDefinition,
    protected readonly RendererInterface $renderer,
    protected readonly EntityTypeManagerInterface $entityTypeManager,
    protected readonly ConfigFactoryInterface $configFactory,
  ) {
    parent::__construct($configuration, $pluginId, $pluginDefinition);
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $pluginId, $pluginDefinition): static {
    return new static(
      $configuration,
      $pluginId,
      $pluginDefinition,
      $container->get('renderer'),
      $container->get('entity_type.manager'),
      $container->get('config.factory')
    );
  }

  /**
   * Resolves the webform submission.
   *
   * @param string $id
   *   The webform ID.
   * @param array $data
   *   The array of WebformSubmissionElement objects.
   * @param \Drupal\graphql\GraphQL\Execution\FieldContext $field
   *   The field context.
   *
   * @return \Drupal\webform\WebformSubmissionInterface|null
   *   The webform submission result or NULL if the webform is not found.
   */
  public function resolve(string $id, array $data, FieldContext $field): ?WebformSubmissionInterface {
    $renderContext = new RenderContext();
    $renderResult = $this->renderer->executeInRenderContext($renderContext, function () use ($id, $data, $field) {
      $webform = $this->getWebformEntityStorage()->load($id);
      if (!$webform instanceof WebformInterface) {
        return NULL;
      }

      $dataElements = array_column($data, 'value', 'key');

      $result = WebformSubmissionEntity::create([
        'webform_id' => $webform->id(),
        'data' => $dataElements,
      ]);

      $result->save();
      $field->addCacheableDependency($webform);

      return $result;
    });

    if (!$renderContext->isEmpty()) {
      $field->addCacheableDependency($renderContext->pop());
    }

    return $renderResult;
  }

  /**
   * Gets the webform entity storage.
   *
   * @return \Drupal\webform\WebformEntityStorageInterface
   *   The webform entity storage.
   */
  protected function getWebformEntityStorage(): WebformEntityStorageInterface {
    return $this->entityTypeManager->getStorage('webform');
  }

  /**
   * Gets a webform setting.
   *
   * @param string $key
   *   The setting key.
   *
   * @return mixed
   *   The setting value.
   */
  protected function getWebformSetting(string $key): mixed {
    return $this->configFactory->get('webform.settings')->get($key);
  }

}
